#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
import os
import shutil
import errno
from stat import (S_IRUSR, S_IWUSR)

from iobase.exception import InvalidArgError
from ionas import confut

# FIXME 部品ファイルパスはpal_configからとる?
import pal_config as config

CRONTAB_STR_FMT_STR = \
        "# %s\n" + \
        "%s %s %s %s %s %s %s\n" + \
        "\n"

WRITE_MODE = "w"


def DEBUG(string):
    #open("/tmp/1", "a").write(string + "\n")
    pass


def __get_crontab_str(entry_name, cmd, user,
        minutes, hour, day, month, week):

    return CRONTAB_STR_FMT_STR % (
            entry_name,
            minutes,
            hour,
            day,
            month,
            week,
            user,
            cmd)


def __add_part_file(crontab_str, entry_name):
    """Add part file."""

    if not os.path.exists(config.CROND_CONF_PART_DIR):
        os.makedirs(config.CROND_CONF_PART_DIR)
    if not os.path.exists(config.CROND_CONF_REAL_DIR):
        os.makedirs(config.CROND_CONF_REAL_DIR)

    tmp_file_path = os.path.join(config.TEMP_DIR, entry_name)
    tmp_file = open(tmp_file_path, WRITE_MODE)
    tmp_file.write(crontab_str)
    tmp_file.close()

    file_path = os.path.join(config.CROND_CONF_PART_DIR, entry_name)
    shutil.move(tmp_file_path, file_path)


def __delete_part_file(entry_name):

    file_path = os.path.join(config.CROND_CONF_PART_DIR, entry_name)

    try:
        os.unlink(file_path)
    except (IOError, OSError), ins:
        if ins.errno == errno.ENOENT:
            pass


def __is_in_range(target, min, max):

    if target == "*":
        return True

    try:
        if int(target) >= min and int(target) <= max:
            return True
    except:
        return False

    return False


def __check_args(entry_name, cmd, user, minutes, hour, day, month, week):

    if not (
        isinstance(entry_name, str) and
        isinstance(cmd, str) and
        isinstance(user, str) and
        isinstance(minutes, str) and
        isinstance(hour, str) and
        isinstance(day, str) and
        isinstance(month, str) and
        isinstance(week, str)):
        raise InvalidArgError


def add_entry(entry_name, cmd, user="nobody",
    minutes="*", hour="*", day="*", month="*", week="*"):
    """ Add or Modify cron entry. """

    __check_args(entry_name, cmd, user, minutes, hour, day, month, week)

    __add_part_file(
        __get_crontab_str(entry_name, cmd, user,
            minutes, hour, day, month, week),
        entry_name)

    part_file_list = [f for f in os.listdir(config.CROND_CONF_PART_DIR) \
            if os.path.isfile(os.path.join(config.CROND_CONF_PART_DIR, f))]
    confut.enable_setting(
            part_file_list,
            [],
            config.CROND_CONF_PART_DIR,
            config.CROND_CONF_REAL_DIR)
    confut.build_config_file(config.CRONTAB_FILE, config.CROND_CONF_REAL_DIR)
    os.chmod(config.CRONTAB_FILE, S_IRUSR | S_IWUSR)


def delete_entry(entry_name):
    """ Delete cron entry. """

    confut.disable_setting(
            [entry_name],
            config.CROND_CONF_REAL_DIR)
    confut.build_config_file(config.CRONTAB_FILE, config.CROND_CONF_REAL_DIR)
    os.chmod(config.CRONTAB_FILE, S_IRUSR | S_IWUSR)

    __delete_part_file(entry_name)


if __name__ == "__main__":
    """ TEST usage.
        PYTHONPATH=../../../test python ctrl.py
    """

    try:
        os.mkdir(config.CROND_CONF_PART_DIR)
    except:
        pass

    CONTENT_FMT_STR = "entry-> %s\ncontents->\n%s"

    print "### Add"
    add_entry("ENTRY_NAME", "COMMAND", user="nobody", minutes="0",
        hour="1", day="2", month="3", week="4")
    add_entry("ENTRY_NAME2", "COMMAND2", minutes="45")
    for name in os.listdir(config.CROND_CONF_PART_DIR):
        file_path = os.path.join(config.CROND_CONF_PART_DIR, name)
        print CONTENT_FMT_STR % (name, open(file_path, "r").read())

    print "### Modify"
    add_entry("ENTRY_NAME", "COMMAND3", user="root", minutes="1",
        hour="2", day="3", month="4", week="5")
    for name in os.listdir(config.CROND_CONF_PART_DIR):
        file_path = os.path.join(config.CROND_CONF_PART_DIR, name)
        print CONTENT_FMT_STR % (name, open(file_path, "r").read())

    print "### Delete"
    delete_entry("ENTRY_NAME")
    for name in os.listdir(config.CROND_CONF_PART_DIR):
        file_path = os.path.join(config.CROND_CONF_PART_DIR, name)
        print CONTENT_FMT_STR % (name, open(file_path, "r").read())

    print "### Error"
    try:
        add_entry("ENTRY_NAME", "COMMAND3", user="root", minutes="60")
    except InvalidArgError:
        print "OK"
    except:
        print "NG"

    shutil.rmtree(config.TEMP_DIR)
