#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""PAL command for firmware settings."""


import os
import tarfile
import re
import libpal.palcmd as palcmd
from pal_info import (Results, ReqTypes)
import pal_config as config
import albg_info


VERSION_RE = "\d+\.\d+"


class PalFirmware(palcmd.PalCmd):
    """Pal firmware implementation class."""

    def __query_current_version(self):
        """Query the current firmware version."""

        with open(config.SYS_FW_VERSION_PATH) as fh:
            return float(fh.read().rstrip())

    def __update_handler(self, entry):
        """Handle update request and set entry to result."""

        if not os.path.exists(entry.get_file()):
            entry.set_result(Results.INVALID_FILE)
            return entry

        try:
            tar = tarfile.open(entry.get_file(), "r")
            ver_file = tar.extractfile(config.FIRMWARE_VERSION_FILE)
            if not ver_file:
                entry.set_result(Results.INVALID_FILE)
                return entry
        except:
            entry.set_result(Results.INVALID_FILE)
            return entry

        new_ver = float(re.findall(VERSION_RE, ver_file.read())[0])
        if new_ver <= self.__query_current_version():
            entry.set_result(Results.OLD_VERSION_ERROR)
            return entry

        result_entry = self.send_entry(entry, albg_info.ALBG_FIRMWARE_PATH)
        return result_entry

    def __init__(self, option=None):
        """Constructor."""

        palcmd.PalCmd.__init__(self, option=option)

        self.set_handler(ReqTypes.UPDATE, self.__update_handler)

#
# Main
#

if __name__ == "__main__":
    PalFirmware(option=palcmd.PalCmdOption()).handle_request()
