#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""PAL command for Notice."""


import libpal.palcmd as palcmd
from pal_info import (ReqTypes, Results)
import pal_config as config
from libpal.crond import ctrl as crond


DAY_XPATH = "report/eventtimelist/eventtime/day"
HOUR_XPATH = "report/eventtimelist/eventtime/hour"
MINUTE_XPATH = "report/eventtimelist/eventtime/minute"

PERIODIC_REPORT = "periodic_report"


class PalNotice(palcmd.PalCmd):
    """Pal Notice command implementation class."""

    def __add_cron_entry(self, entry):

        body = entry.get()

        week = ",".join([node.get_text()
            for node
            in body.get_node_list(DAY_XPATH)])
        hour = body.get_node(HOUR_XPATH).get_text()
        minutes = body.get_node(MINUTE_XPATH).get_text()

        crond.add_entry(
            PERIODIC_REPORT,
            config.CRON_INTERVAL_REPORT_CMD,
            user="root",
            week=week,
            hour=hour,
            minutes=minutes)

        entry.set_result(Results.OK)
        return entry

    def __delete_cron_entry(self, entry):

        crond.delete_entry(PERIODIC_REPORT)

        entry.set_result(Results.OK)
        return entry

    def __apply_handler(self, entry):

        if entry.get().get_node("report").is_enabled():
            return self.__add_cron_entry(entry)
        else:
            return self.__delete_cron_entry(entry)

    def __init__(self, option=None):
        """Constructor."""

        palcmd.PalCmd.__init__(self, option=option)

        self.set_handler(ReqTypes.APPLY, self.__apply_handler)

#
# Main
#

if __name__ == "__main__":
    PalNotice(option=palcmd.PalCmdOption()).handle_request()
