#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""Device Control notify class module.

Define for the Device Control notify class.
"""


import json

from devctrl_class import (RequestData, ResponseData, DevCtrlClass,)
from devctrl_config import (NOTIFY_CONFIG,)


CONFIG_NAME = "config"
CLOUD_NAME = "cloud"
ENABLED_NAME = "enabled"

APPLY_CONFIG_REQUEST = "apply_config"
SAVE_CONFIG_REQUEST = "save_config"
LOAD_CONFIG_REQUEST = "load_config"


class NotifyClass(DevCtrlClass):
    """Class for the notify class."""

    def _apply_handler(self, request_data): 
        """Apply handler."""

        response_data = ResponseData(request_data)
        try:
            config = json.load(open(NOTIFY_CONFIG))
            if request_data[CONFIG_NAME][CLOUD_NAME][ENABLED_NAME]:
                config[CLOUD_NAME][ENABLED_NAME] = True
            else:
                config[CLOUD_NAME][ENABLED_NAME] = False

            json.dump(config, open(NOTIFY_CONFIG, "w"))
            response_data.result = 0
        except:
            response_data.result = 1
        return str(response_data)

    def handle_request(self, request):
        """Handle notify class requests."""

        try:
            request_data = RequestData(request)
        except:
            response_data = ResponseData()
            response_data.result = 1
            return str(response_data)

        try:
            if request_data.request == APPLY_CONFIG_REQUEST:
                return self._apply_handler(request_data)
            elif request_data.request == SAVE_CONFIG_REQUEST:
                return self._json_save_handler(request_data, CONFIG_NAME,
                                                NOTIFY_CONFIG)
            elif request_data.request == LOAD_CONFIG_REQUEST:
                return self._json_load_handler(request_data, CONFIG_NAME,
                                                NOTIFY_CONFIG)
        except:
            pass

        response_data = ResponseData(request_data)
        response_data.result = 1
        return str(response_data)


def new_handler_class():
    """Return notify handler class instance."""

    return NotifyClass()
