#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""Do something for uninstalling the remotelink3."""

from ionas.request import (Request, Entry)
from iobase.exception import (IoBaseException,)
from fal_info import (FAL_FEATURES_PATH, Results, ReqTypes)

import manualctrl


FEATURES_REQ_XML = "<features><functionlist>" + \
                "<function>remotelink3</function></functionlist></features>"


class RequestError(IoBaseException):
    """Exception Class for request error."""

    pass


def __send_request(target, type, body="", err_msg=""):
    """Send request entry to "target"."""

    entry = Entry(type, body)
    result_entry = Request(target).request(entry).get_entry()
    if (result_entry.get_result() != Results.OK and
        result_entry.get_result() != Results.FILE_NOT_FOUND):

        raise RequestError("%s - (%s)" % (err_msg, result_entry.get_result()))

    try:
        return result_entry.get()
    except:
        return ""


def delete_remotelink3_feature():
    """Delete Remote Link 3 feature info."""

    try:
        __send_request(FAL_FEATURES_PATH, ReqTypes.DELETE, FEATURES_REQ_XML)
    except:
        # Ignore error
        pass

#
# Main
#


delete_remotelink3_feature()
manualctrl.uninstall()
