#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
import os
import sys
import stat
import time
import signal
import rapscomm

# initalize
c_comm   = rapscomm.rapscomm()
n_result = c_comm.RESULT.RESULT_NO_ERROR

# output exit code
def exitout():
    sys.stdout.write(str(n_result))
    sys.stdout.write("\n")
    sys.exit(0)

# set signal function
def sigfunc(num, frame):
    n_result = c_comm.RESULT.RESULT_ERROR
    exitout()

# localtime to UTC
def ltm2utc(ltime):
    rettime = ltime    
    if time.daylight:
        rettime -= time.altzone
    else:
        rettime -= time.timezone
    return rettime

signal.signal( signal.SIGTERM, sigfunc)

# parameter check
if len(sys.argv) < 5:
    n_result = c_comm.RESULT.RESULT_INVALID_OPTION
    exitout()

# get parameter
i_uid = int(sys.argv[1])
i_gid = int(sys.argv[2])
s_tim = sys.argv[3]
s_src = sys.argv[4]

# set time stamp
try:
    d_atime = os.stat(s_src)[stat.ST_ATIME]
    d_mtime = time.mktime( time.strptime( s_tim, '%Y%m%d%H%M%S') )
    d_mtime = ltm2utc(d_mtime)
    os.utime( s_src, (d_atime, d_mtime) )
except:
    pass

exitout()

