# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from command import Command, get_message, logging
from ionas.request import Entry, XmlConfNode
from fal_info import Results
import datetime
import time


not_given = object()


epoch_to_datetime = lambda e: datetime.datetime(*time.localtime(e)[:6])


def _xmlnode__get_value(xmlnode, xpath='.', default=None, cast=lambda v:v):
    try:
        empty = object()
        value = xmlnode.get_value(xpath, empty)
        if value is empty:
            return default
        return cast(value)
        #return xmlnode.get_value(xpath, default)
    except:
        return default


def _xmlnode__is_true(xmlnode, xpath='.', default=bool(0)):
    try:
        return xmlnode.get_value(xpath) == 'true'
    except:
        return default


class FALRemoteLink3(Command):

    try:
        from fal_info import FAL_REMOTELINK3_PATH as COMMAND
    except:
        COMMAND_NAME = "fal_remotelink3.py"

    ###########################################################################
    # public                                                                  #

    def get_userpolicyflag(self):
        response = {
            "entry": None,
            "result": None,
            "message": None,
            "data": {},
            "success": False,
        }

        try:
            entry = Entry("get_userpolicyflag", "<remotelink3/>")

            response["entry"] = self.request(entry)

            response["result"] = response["entry"].get_result()

            messages = {
            }
            constname, arglist = messages.get(response["result"], messages.get("default", ["", {}]))
            if constname:
                self._set_message(constname, arglist=arglist, log=True)
                response["message"] = self.get_message()

            remotelink3_node = response["entry"].get()
            response["data"] = {
                "flag": bool(int(remotelink3_node.get_value("userpolicyflag", "0"))),
            }

            response["success"] = response["result"] in [Results.OK]
        except:
            pass

        return response

    def set_userpolicyflag(self, flag=True):
        response = {
            "entry": None,
            "result": None,
            "message": None,
            "data": {},
            "success": False,
        }

        try:
            entry = Entry("set_userpolicyflag", "<remotelink3><userpolicyflag/></remotelink3>")
            entry.get().get_node("userpolicyflag").set_text(str(int(flag)))

            response["entry"] = self.request(entry)

            response["result"] = response["entry"].get_result()

            messages = {
                Results.OK: ("MSG_REMOTELINK3_CHANGE_FLAG", {}),
                "default": ("ERR_REMOTELINK3_CHANGE_FLAG", {}),
            }
            constname, arglist = messages.get(response["result"], messages.get("default", ["", {}]))
            if constname:
                self._set_message(constname, arglist=arglist, log=True)
                response["message"] = self.get_message()

            remotelink3_node = response["entry"].get()
            response["data"] = {
                "flag": bool(int(remotelink3_node.get_value("userpolicyflag", "0"))),
            }

            response["success"] = response["result"] in [Results.OK]
        except:
            pass

        return response

    def load(self):
        response = {
            "entry": None,
            "result": None,
            "message": None,
            "data": {},
            "success": False,
        }

        try:
            ###################################################################
            # messages, entry                                                 #
            messages = {
            }
            entry = Entry("load", """<remotelink3/>""")
            #                                                                 #
            ###################################################################

            response["entry"] = self.request(entry)

            response["result"] = response["entry"].get_result()

            constname, arglist = messages.get(response["result"], messages.get("default", ["", {}]))
            if constname:
                self._set_message(constname, arglist=arglist, log=True)
                response["message"] = self.get_message()

            ###################################################################
            # response["data"]                                                #

            remotelink3_node = response["entry"].get()
            response["data"] = {
                "terms": {
                    "agree": _xmlnode__is_true(remotelink3_node, "terms/@agree"),
                },
                "remoteaccess": {
                    "enabled": _xmlnode__is_true(remotelink3_node, "remoteaccess/@enable"),
                    "timelimit": {
                        "enabled": _xmlnode__is_true(remotelink3_node, "remoteaccess/timelimit/@enable"),
                    },
                    "clientlimit": {
                        "enabled": _xmlnode__is_true(remotelink3_node, "remoteaccess/clientlimit/@enable"),
                    },
                    "sharelist": [{
                        "name": share_node.get_value("name"),
                        "enabled": _xmlnode__is_true(share_node, "@enable"),
                        "client": _xmlnode__is_true(share_node, "@client"),
                    } for share_node in remotelink3_node.get_node_list("remoteaccess/sharelist/share")],
                    "refreshconnection": {
                        "enabled": _xmlnode__is_true(remotelink3_node, "remoteaccess/refreshconnection/@enable"),
                        "period": _xmlnode__get_value(remotelink3_node, xpath="remoteaccess/refreshconnection/period", default=7, cast=int),
                    },
                },
                "connection": {
                    "port1": remotelink3_node.get_value('connection/portlist/port[@type="http"]'),
                    "port2": remotelink3_node.get_value('connection/portlist/port[@type="https"]'),
                    "upnp": {
                        "enabled": _xmlnode__is_true(remotelink3_node, "connection/upnp/@enable"),
                        "remap": _xmlnode__is_true(remotelink3_node, "connection/upnp/@remap"),
                        "port1": remotelink3_node.get_value('connection/upnp/portlist/port[@type="http"]'),
                        "port2": remotelink3_node.get_value('connection/upnp/portlist/port[@type="https"]'),
                    },
                },
                "ioportal": {
                    "account": {
                        "name": remotelink3_node.get_value("ioportal/account/name"),
                        "password": remotelink3_node.get_value("ioportal/account/password"),
                    },
                    "product": {
                        "serialnumber": remotelink3_node.get_value("ioportal/product/serialnumber"),
                    },
                },
            }
            #                                                                 #
            ###################################################################

            response["success"] = response["result"] in [Results.OK]
        except:
            pass

        return response

    def query(self):
        response = {
            "entry": None,
            "result": None,
            "message": None,
            "data": {},
            "success": False,
        }

        try:
            ###################################################################
            # messages, entry  
            entry = Entry("query", """<remotelink3/>""")
            #                                                                 #
            ###################################################################

            response["entry"] = self.request(entry)

            response["result"] = response["entry"].get_result()

            ###################################################################
            # response["data"]                                                #
            remotelink3_node = response["entry"].get()
            response["data"] = {
                "touserver": {
                    "status": remotelink3_node.get_value("touserver/statusinfo/status"),
                    "code": remotelink3_node.get_value("touserver/statusinfo/code"),
                },
            }
            #                                                                 #
            ###################################################################

            response["success"] = response["result"] in [Results.OK]
        except:
            pass

        return response

    def apply(self, config, log=True, messages=not_given):
        response = {
            "entry": None,
            "result": None,
            "message": None,
            "data": {},
            "success": False,
        }

        try:
            ###################################################################
            # default_messages, entry                                         #
            default_messages = {
                Results.OK: ("MSG_RL3_CHANGE", {}),
                Results.AUTH_ERROR: ("ERR_RL3_IOPORTAL_AUTH", {}),
                Results.INVALID_DATA: ("ERR_RL3_IOPORTAL_INVALID_MAC_SN", {}),
                Results.CONNECT_ERROR: ("ERR_RL3_SERVER_CONNECT", {}),
                "default": ("ERR_RL3_CHANGE", {}),
            }
            entry = Entry("apply", """<remotelink3/>""")
            remotelink3_node = entry.get()
            if "terms" in config:
                remotelink3_node.append_child("terms", attrs={"agree": str(config["terms"]["agree"]).lower()})
            if "ioportal" in config:
                remotelink3_node.append_child("ioportal")
                ioportal_node = remotelink3_node.get_node("ioportal")
                ioportal_node.append_child("account")
                account_node = ioportal_node.get_node("account")
                account_node.append_child("name", config["ioportal"]["account"]["name"])
                account_node.append_child("password", config["ioportal"]["account"]["password"])
                ioportal_node.append_child("product")
                product_node = ioportal_node.get_node("product")
                product_node.append_child("serialnumber", config["ioportal"]["product"]["serialnumber"])
            if "remoteaccess" in config:
                remotelink3_node.append_child("remoteaccess")
                remoteaccess_node = remotelink3_node.get_node("remoteaccess")
                remoteaccess_node.enable(config["remoteaccess"]["enabled"])
                remoteaccess_node.append_child("timelimit")
                timelimit_node = remoteaccess_node.get_node("timelimit")
                timelimit_node.enable(config["remoteaccess"]["timelimit"]["enabled"])
                remoteaccess_node.append_child("clientlimit")
                clientlimit_node = remoteaccess_node.get_node("clientlimit")
                clientlimit_node.enable(config["remoteaccess"]["clientlimit"]["enabled"])
                remoteaccess_node.append_child("sharelist")
                sharelist_node = remoteaccess_node.get_node("sharelist")
                for share in config["remoteaccess"]["sharelist"]:
                    share_node = XmlConfNode("<share/>")
                    share_node.set_attrs({
                        "enable": str(share["enabled"]).lower(),
                        "client": str(share["client"]).lower(),
                    })
                    share_node.append_child("name", share["name"])
                    sharelist_node.append_child(share_node)
                remoteaccess_node.append_child("refreshconnection", attrs={
                    "enable": str(config["remoteaccess"]["refreshconnection"]["enabled"]).lower(),
                })
                refreshconnection_node = remoteaccess_node.get_node("refreshconnection")
                refreshconnection_node.append_child("period", text=str(config["remoteaccess"]["refreshconnection"]["period"]))
            if "connection" in config:
                remotelink3_node.append_child("connection")
                connection_node = remotelink3_node.get_node("connection")
                connection_node.append_child("portlist")
                portlist_node = connection_node.get_node("portlist")
                portlist_node.append_child("port", attrs={"type": "http"}, text=str(config["connection"]["port1"]))
                portlist_node.append_child("port", attrs={"type": "https"}, text=str(config["connection"]["port2"]))
                connection_node.append_child("upnp", attrs={
                    "enable": str(config["connection"]["upnp"]["enabled"]).lower(),
                    "remap": str(config["connection"]["upnp"]["remap"]).lower(),
                })
                upnp_node = connection_node.get_node("upnp")
                upnp_node.append_child("portlist")
                portlist_node = upnp_node.get_node("portlist")
                portlist_node.append_child("port", attrs={"type": "http"}, text=str(config["connection"]["upnp"]["port1"]))
                portlist_node.append_child("port", attrs={"type": "https"}, text=str(config["connection"]["upnp"]["port2"]))
            #                                                                 #
            ###################################################################

            if messages is not_given:
                messages = default_messages

            response["entry"] = self.request(entry)

            response["result"] = response["entry"].get_result()

            constname, arglist = messages.get(response["result"], messages.get("default", ["", {}]))
            if constname:
                self._set_message(constname, arglist=arglist, log=log)
                response["message"] = self.get_message()

            ###################################################################
            # response["data"]                                                #
            remotelink3_node = entry.get()
            response["data"] = {
            }
            #                                                                 #
            ###################################################################

            response["success"] = response["result"] in [Results.OK]
        except:
            pass

        return response

    def get_pincode(self):
        response = {
            "entry": None,
            "result": None,
            "message": None,
            "data": {},
            "success": False,
        }

        try:
            entry = Entry("get_pincode", """<remotelink3/>""")

            response["entry"] = self.request(entry)

            response["result"] = response["entry"].get_result()

            messages = {
            }
            constname, arglist = messages.get(response["result"], messages.get("default", ["", {}]))
            if constname:
                self._set_message(constname, arglist=arglist, log=True)
                response["message"] = self.get_message()

            remotelink3_node = response["entry"].get()
            response["data"] = {
                "pincode": remotelink3_node.get_value("pincode")
            }

            response["success"] = response["result"] in [Results.OK]
        except:
            pass

        return response

    def change_pincode(self):
        response = {
            "entry": None,
            "result": None,
            "message": None,
            "data": {},
            "success": False,
        }

        try:
            entry = Entry("change_pincode", """<remotelink3/>""")

            response["entry"] = self.request(entry)

            response["result"] = response["entry"].get_result()

            messages = {
                Results.OK: ("MSG_RL3_CHANGE_PINCODE", {}),
                Results.CONNECT_ERROR: ("ERR_RL3_SERVER_CONNECT", {}),
                "default": ("ERR_RL3_CHANGE_PINCODE", {}),
            }
            constname, arglist = messages.get(response["result"], messages.get("default", ["", {}]))
            if constname:
                self._set_message(constname, arglist=arglist, log=True)
                response["message"] = self.get_message()

            remotelink3_node = entry.get()
            response["data"] = {
                "pincode": remotelink3_node.get_value("pincode")
            }

            response["success"] = response["result"] in [Results.OK]
        except:
            pass

        return response

    def create_qrcode(self, pincode, directory):
        response = {
            "entry": None,
            "result": None,
            "message": None,
            "data": {},
            "success": False,
        }

        try:
            ###################################################################
            # messages, entry                                                 #
            messages = {
            }
            entry = Entry("create_qrcode", """<remotelink3><qrcode/></remotelink3>""")
            qrcode_node = entry.get().get_node("qrcode")
            qrcode_node.append_child("pincode", pincode)
            qrcode_node.append_child("directory", directory)
            #                                                                 #
            ###################################################################

            response["entry"] = self.request(entry)

            response["result"] = response["entry"].get_result()

            constname, arglist = messages.get(response["result"], messages.get("default", ["", {}]))
            if constname:
                self._set_message(constname, arglist=arglist, log=True)
                response["message"] = self.get_message()

            ###################################################################
            # response["data"]                                                #
            remotelink3_node = response["entry"].get()
            response["data"] = {
                "filename": remotelink3_node.get_value("qrcode/filename")
            }
            #                                                                 #
            ###################################################################

            response["success"] = response["result"] in [Results.OK]
        except:
            pass

        return response

    def get_timelimit_list(self):
        response = {
            "entry": None,
            "result": None,
            "message": None,
            "data": {},
            "success": False,
        }

        try:
            ###################################################################
            # messages, entry                                                 #
            messages = {
            }
            entry = Entry("get_timelimit_list", """<remotelink3/>""")
            #                                                                 #
            ###################################################################

            response["entry"] = self.request(entry)

            response["result"] = response["entry"].get_result()

            constname, arglist = messages.get(response["result"], messages.get("default", ["", {}]))
            if constname:
                self._set_message(constname, arglist=arglist, log=True)
                response["message"] = self.get_message()

            ###################################################################
            # response["data"]                                                #
            remotelink3_node = response["entry"].get()
            response["data"] = {
                "timelimitlist": [{
                    "pincode": timelimit_node.get_value("pincode"),
                    "pathlist": [path_node.get_text() for path_node in timelimit_node.get_node_list("pathlist/path")],
                    #"expire": int(timelimit_node.get_value("expire/time") or "0"),
                    "expire": _xmlnode__get_value(timelimit_node, xpath="expire/time", default=None, cast=int),
                    "clientmax": int(timelimit_node.get_value("clientmax") or "0"),
                    "accessmax": int(timelimit_node.get_value("accessmax") or "0"),
                    "username": timelimit_node.get_value("registerinfo/username"),
                    "registerinfo": {
                        "username": timelimit_node.get_value("registerinfo/username"),
                        #"time": timelimit_node.get_value("registerinfo/time"),
                        "time": _xmlnode__get_value(timelimit_node, xpath="registerinfo/time", default=None, cast=int),
                    },
                    "clients": [node.get_text() for node in timelimit_node.get_node_list("clientidlist/clientid")],
                    "accesscounts": dict([(node.get_text(), int(node.get_attr_value("accesscount"))) for node in timelimit_node.get_node_list("accessinfo/pathlist/path")]),
                } for timelimit_node in remotelink3_node.get_node_list("timelimitlist/timelimit")],
            }
            #                                                                 #
            ###################################################################

            response["success"] = response["result"] in [Results.OK]
        except:
            pass

        return response

    def delete_timelimit(self, pincode, log=True):
        response = {
            "entry": None,
            "result": None,
            "message": None,
            "data": {},
            "success": False,
        }

        try:
            ###################################################################
            # messages, entry                                                 #
            messages = {
                Results.OK: ("MSG_RL3_DELETE_TIMELIMIT_ENTRY", {}),
                "default": ("ERR_RL3_DELETE_TIMELIMIT_ENTRY", {}),
            }
            entry = Entry("delete_timelimit", """<remotelink3><timelimit/></remotelink3>""")
            entry.get().get_node("timelimit").append_child("pincode", pincode)
            #                                                                 #
            ###################################################################

            response["entry"] = self.request(entry)

            response["result"] = response["entry"].get_result()

            constname, arglist = messages.get(response["result"], messages.get("default", ["", {}]))
            if constname:
                self._set_message(constname, arglist=arglist, log=log)
                response["message"] = self.get_message()

            ###################################################################
            # response["data"]                                                #
            remotelink3_node = response["entry"].get()
            response["data"] = {
            }
            #                                                                 #
            ###################################################################

            response["success"] = response["result"] in [Results.OK]
        except:
            pass

        return response

    def add_clientlimit(self, label="", emailaddress=""):
        response = {
            "entry": None,
            "result": None,
            "message": None,
            "data": {},
            "success": False,
        }

        try:
            ###################################################################
            # messages, entry                                                 #
            messages = {
                Results.OK: ("MSG_RL3_CREATE_CLIENTLIMIT_PINCODE", {}),
                "default": ("ERR_RL3_CREATE_CLIENTLIMIT_PINCODE", {}),
            }
            entry = Entry("add_clientlimit", """<remotelink3><clientlimit/></remotelink3>""")
            entry.get().get_node("clientlimit").append_child("name", label)
            entry.get().get_node("clientlimit").append_child("mailaddr", emailaddress)
            #                                                                 #
            ###################################################################

            response["entry"] = self.request(entry)

            response["result"] = response["entry"].get_result()

            constname, arglist = messages.get(response["result"], messages.get("default", ["", {}]))
            if constname:
                self._set_message(constname, arglist=arglist, log=True)
                response["message"] = self.get_message()

            ###################################################################
            # response["data"]                                                #
            remotelink3_node = response["entry"].get()
            response["data"] = {
                "pincode": remotelink3_node.get_value("clientlimit/pincode"),
            }
            #                                                                 #
            ###################################################################

            response["success"] = response["result"] in [Results.OK]
        except:
            pass

        return response

    def modify_clientlimit(self, pincode, label="", emailaddress=""):
        response = {
            "entry": None,
            "result": None,
            "message": None,
            "data": {},
            "success": False,
        }

        try:
            ###################################################################
            # messages, entry                                                 #
            messages = {
                Results.OK: ("MSG_RL3_CHANGE_CLIENTLIMIT_ENTRY", {}),
                "default": ("ERR_RL3_CHANGE_CLIENTLIMIT_ENTRY", {}),
            }
            entry = Entry("modify_clientlimit", """<remotelink3><clientlimit/></remotelink3>""")
            entry.get().get_node("clientlimit").append_child("pincode", pincode)
            entry.get().get_node("clientlimit").append_child("name", label)
            entry.get().get_node("clientlimit").append_child("mailaddr", emailaddress)
            #                                                                 #
            ###################################################################

            response["entry"] = self.request(entry)

            response["result"] = response["entry"].get_result()

            constname, arglist = messages.get(response["result"], messages.get("default", ["", {}]))
            if constname:
                self._set_message(constname, arglist=arglist, log=True)
                response["message"] = self.get_message()

            ###################################################################
            # response["data"]                                                #
            remotelink3_node = response["entry"].get()
            response["data"] = {
            }
            #                                                                 #
            ###################################################################

            response["success"] = response["result"] in [Results.OK]
        except:
            pass

        return response

    def delete_clientlimit(self, pincode, log=True):
        response = {
            "entry": None,
            "result": None,
            "message": None,
            "data": {},
            "success": False,
        }

        try:
            ###################################################################
            # messages, entry                                                 #
            messages = {
                Results.OK: ("MSG_RL3_DELETE_CLIENTLIMIT_ENTRY", {}),
                "default": ("ERR_RL3_DELETE_CLIENTLIMIT_ENTRY", {}),
            }
            entry = Entry("delete_clientlimit", """<remotelink3><clientlimit/></remotelink3>""")
            entry.get().get_node("clientlimit").append_child("pincode", pincode)
            #                                                                 #
            ###################################################################

            response["entry"] = self.request(entry)

            response["result"] = response["entry"].get_result()

            constname, arglist = messages.get(response["result"], messages.get("default", ["", {}]))
            if constname:
                self._set_message(constname, arglist=arglist, log=log)
                response["message"] = self.get_message()

            ###################################################################
            # response["data"]                                                #
            remotelink3_node = response["entry"].get()
            response["data"] = {
            }
            #                                                                 #
            ###################################################################

            response["success"] = response["result"] in [Results.OK]
        except:
            pass

        return response

    def get_clientlimit_list(self):
        response = {
            "entry": None,
            "result": None,
            "message": None,
            "data": {},
            "success": False,
        }

        try:
            ###################################################################
            # messages, entry                                                 #
            messages = {
            }
            entry = Entry("get_clientlimit_list", """<remotelink3/>""")
            #                                                                 #
            ###################################################################

            response["entry"] = self.request(entry)

            response["result"] = response["entry"].get_result()

            constname, arglist = messages.get(response["result"], messages.get("default", ["", {}]))
            if constname:
                self._set_message(constname, arglist=arglist, log=True)
                response["message"] = self.get_message()

            ###################################################################
            # response["data"]                                                #
            remotelink3_node = response["entry"].get()
            response["data"] = {
                "clientlimitlist": [{
                    "label": clientlimit_node.get_value("name"),
                    "emailaddress": clientlimit_node.get_value("mailaddr"),
                    "clientid": clientlimit_node.get_value("clientid"),
                    "pincode": clientlimit_node.get_value("pincode"),
                    "registerinfo": {
                        "username": clientlimit_node.get_value("registerinfo/username"),
                        #"time": int(clientlimit_node.get_value("registerinfo/time", None)),
                        "time": _xmlnode__get_value(clientlimit_node, xpath="registerinfo/time", default=None, cast=int),
                    },
                    "logininfo": {
                        #"time": clientlimit_node.get_value("logininfo/time"),
                        "time": _xmlnode__get_value(clientlimit_node, xpath="logininfo/time", default=None, cast=int),
                    },
                } for clientlimit_node in remotelink3_node.get_node_list("clientlimitlist/clientlimit")],
            }
            #                                                                 #
            ###################################################################

            response["success"] = response["result"] in [Results.OK]
        except:
            pass

        return response

    def send_pincode(self, emailaddress, pincode, messages=not_given):
        response = {
            "entry": None,
            "result": None,
            "message": None,
            "data": {},
            "success": False,
        }

        try:
            ###################################################################
            # default_messages, entry                                         #
            default_messages = {
                Results.OK: ("MSG_RL3_SEND_CLIENTLIMIT_PINCODE", {"mail_address": emailaddress}),
                "default": ("ERR_RL3_SEND_CLIENTLIMIT_PINCODE", {"mail_address": emailaddress}),
            }
            entry = Entry("send_pincode", """<remotelink3/>""")
            entry.get().append_child("pincode", pincode)
            entry.get().append_child("mailaddr", emailaddress)
            #                                                                 #
            ###################################################################

            if messages is not_given:
                messages = default_messages

            response["entry"] = self.request(entry)

            response["result"] = response["entry"].get_result()

            constname, arglist = messages.get(response["result"], messages.get("default", ["", {}]))
            if constname:
                self._set_message(constname, arglist=arglist, log=True)
                response["message"] = self.get_message()

            ###################################################################
            # response["data"]                                                #
            remotelink3_node = response["entry"].get()
            response["data"] = {
            }
            #                                                                 #
            ###################################################################

            response["success"] = response["result"] in [Results.OK]
        except:
            pass

        return response

    def obtain_clients(self):
        response = {
            "entry": None,
            "result": None,
            "message": None,
            "data": {},
            "success": False,
        }
        try:
            messages = {
            }
            entry = Entry("get_connection_list", """<remotelink3/>""")
            response["entry"] = self.request(entry)
            response["result"] = response["entry"].get_result()
            constname, arglist = messages.get(
                response["result"],
                messages.get("default", ["", {}]),
            )
            remotelink3_node = response["entry"].get()
            response["data"] = {
                "connections": [{
                    "id": connection_node.get_node("hashedid").get_text(),
                    "cid": connection_node.get_node("clientid").get_text(),
                    "alloweddenied": connection_node.get_node("status").get_text(),
                    "connected_username": connection_node.get_node('lastinfo/name').get_text(),
                    "connected_datetime": connection_node.get_node("lastinfo/time"),
                    "connected_datetime": epoch_to_datetime(_xmlnode__get_value(connection_node, xpath="lastinfo/time", default=0, cast=int)),
                } for connection_node in remotelink3_node.get_node_list("connectionlist/connection")],
            }
            response["success"] = response["result"] in [Results.OK]
        except Exception as err:
            pass
        return response;

    def allow_connection(self, id):
        messages = {
            Results.OK: ('MSG_RL3_ALLOW_CONNECTION', {}),
            'default': ('ERR_RL3_ALLOW_CONNECTION', {}),
        }
        return self._update_connection_alloweddenied(id, 'allowed', messages)

    def deny_connection(self, id):
        messages = {
            Results.OK: ('MSG_RL3_DENY_CONNECTION', {}),
            'default': ('ERR_RL3_DENY_CONNECTION', {}),
        }
        return self._update_connection_alloweddenied(id, 'denied', messages)

    def _update_connection_alloweddenied(self,
                                         id,
                                         allowed_or_denied,
                                         messages):
        response = {
            "entry": None,
            "result": None,
            "message": None,
            "data": {},
            "success": False,
        }
        try:
            entry = Entry("modify_connection", """<remotelink3><connection/></remotelink3>""")
            entry.get().get_node('connection').append_child("hashedid", id)
            entry.get().get_node('connection').append_child("status", allowed_or_denied)

            response["entry"] = self.request(entry)

            response["result"] = response["entry"].get_result()

            constname, arglist = messages.get(
                response["result"],
                messages.get("default", ["", {}]),
            )
            if constname:
                self._set_message(constname, arglist=arglist, log=True)
                response["message"] = self.get_message()

            remotelink3_node = response["entry"].get()
            response["data"] = {
            }

            response["success"] = response["result"] in [Results.OK]
        except:
            pass

        return response

    def remove_connection(self, id):
        response = {
            "entry": None,
            "result": None,
            "message": None,
            "data": {},
            "success": False,
        }
        try:
            messages = {
                Results.OK: ('MSG_RL3_DELETE_CONNECTION_ENTRY', {}),
                'default': ('ERR_RL3_DELETE_CONNECTION_ENTRY', {}),
            }
            entry = Entry("delete_connection", """<remotelink3><connection/></remotelink3>""")
            entry.get().get_node('connection').append_child("hashedid", id)

            response["entry"] = self.request(entry)

            response["result"] = response["entry"].get_result()

            constname, arglist = messages.get(
                response["result"],
                messages.get("default", ["", {}]),
            )
            if constname:
                self._set_message(constname, arglist=arglist, log=True)
                response["message"] = self.get_message()

            remotelink3_node = response["entry"].get()
            response["data"] = {
            }

            response["success"] = response["result"] in [Results.OK]
        except:
            pass

        return response


if __name__ == "__main__":
    pass
    falremotelink3 = FALRemoteLink3()
    print falremotelink3.get_pincode()
