# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from django.utils.translation import ugettext as _
from django import forms
from django.forms.formsets import formset_factory
from django.conf import settings
from project.view import FuncView
from .. fields import RemoteLink3ExpirationEntrysMultipleCheckboxField
import os
import datetime
import time
from project.fal.remotelink3 import FALRemoteLink3, get_message, logging


def _make_qrcode(pincode):
    try:
        if pincode:
            directory = os.path.join(settings.MEDIA_ROOT, "img", "pkg", "remotelink3")
            filename = FALRemoteLink3().create_qrcode(pincode, directory)["data"]["filename"]
            return os.path.join(settings.MEDIA_URL, "img", "pkg", "remotelink3", filename)
        return None
    except:
        return None


def _delete_pincode(pincode, log=True):
    try:
        r = FALRemoteLink3().delete_timelimit(pincode, log=log)
        success = r['success']
        message = r['message']
        success = success if isinstance(success, bool) else False
        message = message if isinstance(message, basestring) else u''
        return success, message
    except:
        return False, u''


def _get_entrys():
    entrys = []
    try:
        datetime_format = '%Y/%m/%d %H:%M:%S'  # FIXME: locale
        expire_format = '%Y/%m/%d'  # FIXME: locale
        for e in FALRemoteLink3().get_timelimit_list()["data"]["timelimitlist"]:
            t = e['registerinfo']['time']
            if t: t = datetime.datetime(*time.localtime(t)[:6]).strftime(datetime_format)
            entrys.append({
                'pincode': e['pincode'],
                'paths': e['pathlist'],
                #'expire': datetime.datetime(*time.localtime(e['expire'])[:6]),
                'expire': datetime.datetime(*time.localtime(e['expire'])[:6]).strftime(expire_format),
                'client_max_num': e['clientmax'],
                'access_max_num': e['accessmax'],
                'username': e['username'],
                'registertime': t,
            })
        return entrys
    except:
        pass
    return entrys


def _timelimit_enabled():
    try:
        return FALRemoteLink3().load()["data"]['remoteaccess']['timelimit']['enabled']
    except:
        return False


class RemoteLink3ExpirationForm(forms.Form):

    def __init__(self, entrys, *args, **kwargs):
        super(RemoteLink3ExpirationForm, self).__init__(*args, **kwargs)
        choices = [(entry["pincode"], entry["pincode"]) for entry in entrys]
        self.fields["entrys"] = RemoteLink3ExpirationEntrysMultipleCheckboxField(choices=choices)


class RemoteLink3ExpirationView(FuncView):

    def __deco_view(view_func):
        def _view_func(self, request):
            template, dictionary = view_func(self, request)
            return {
                self.KEY_TEMPLATE: template,
                self.KEY_TEMPLATE_DIC: dictionary,
            }
        return _view_func

    views = ['delete']

    @__deco_view
    def view(self, request):
        entrys = _get_entrys()
        form = RemoteLink3ExpirationForm(entrys)
        for e, cb in zip(entrys, form["entrys"]):
            e["cb"] = cb

        template = os.path.join(self.TEMPLATES, 'entries.html')
        dictionary = {
            'form': form,
            'entrys': entrys,
            'timelimit_enabled': _timelimit_enabled(),
        }
        return template, dictionary

    @__deco_view
    def view_delete(self, request):
        entrys = _get_entrys()
        form = RemoteLink3ExpirationForm(entrys, data=request.POST)
        for e, cb in zip(entrys, form["entrys"]):
            e["cb"] = cb

        with self.processing(constname='ERR_RL3_STATE_CHANGE') as proc:
            proc.break_if_state_change()
            if form.is_valid():
                result = True
                for pincode in form.cleaned_data['entrys']:
                    _result, _message = _delete_pincode(pincode, log=False)
                    result = result and _result
                    #self.set_message(_message)
                proc.result(result)

                if proc.result():
                    name = 'MSG_RL3_DELETE_TIMELIMIT_ENTRY'
                else:
                    name = 'ERR_RL3_DELETE_TIMELIMIT_ENTRY'
                logging(name)
                self.set_message(get_message(name))

        entrys = _get_entrys()
        form = RemoteLink3ExpirationForm(entrys)
        for e, cb in zip(entrys, form["entrys"]):
            e["cb"] = cb

        template = os.path.join(self.TEMPLATES, 'entries.html')
        dictionary = {
            'form': form,
            'entrys': entrys,
            'timelimit_enabled': _timelimit_enabled(),
        }
        return template, dictionary


class RemoteLink3TemporaryAccessEntryDetailView(FuncView):

    TITLE = _(u"remotelink3 temporary access")

    def __deco_view(view_func):
        def _view_func(self, request, *args, **kwargs):
            template, dictionary = view_func(self, request, *args, **kwargs)
            return {
                self.KEY_TEMPLATE: template,
                self.KEY_TEMPLATE_DIC: dictionary,
            }
        return _view_func

    def __get_entry_info(self, pincode):
        try:
            expire_format = '%Y/%m/%d'  # FIXME: locale
            for e in FALRemoteLink3().get_timelimit_list()['data']['timelimitlist']:
                if e['pincode'] == pincode:
                    return {
                        'pincode': pincode,
                        'qrcode': _make_qrcode(pincode),
                        'paths': e['pathlist'],
                        #'expire': e['expire'],
                        'expire': datetime.datetime(*time.localtime(e['expire'])[:6]).strftime(expire_format),
                        'client_max_num': e['clientmax'],
                        'access_max_num': e['accessmax'],
                        'clients': e['clients'],
                        'accesscounts': e['accesscounts'],
                    }
            return None
        except:
            return None

    @__deco_view
    def view(self, request, pincode=''):
        entry_info = self.__get_entry_info(pincode)
        if entry_info is None:
            pass  # FIXME

        template = os.path.join(self.TEMPLATES, 'entry.html')
        dictionary = {
            'entry_info': entry_info,
            'url': '../',
        }
        return template, dictionary
