# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from django.utils.translation import ugettext as _
from django import forms
from multiplecheckboxfield import MultipleCheckboxField
import re


empty = object()


###############################################################################
#                                                                             #
from project.forms.fields.extras.enable import EnableField
class EnabledRadioField(EnableField):
    pass


from project.forms.fields.extras.portnum import PortNumberField
from project.forms.fields.extras.validators.fal import FALValidator
from fal_validinfo import LocalPortNumber
class LocalPortNumberTextField(PortNumberField):

    SERVICE = None

    def __init__(self, *args, **kwargs):
        if self.SERVICE:
            validator = type('LocalPortNumberValidator',
                             (FALValidator,),
                             {
                                 'validator': LocalPortNumber.VALIDATOR,
                                 'option': {
                                     'service': self.SERVICE,
                                     'exempt_null': 'true',  # FIXME
                                 },
                             })
            self.default_validators.append(validator)
        super(LocalPortNumberTextField, self).__init__(*args, **kwargs)


from project.forms.fields.extras.portnum import RemotePortNumberField
class RemotePortNumberTextField(RemotePortNumberField):
    pass


class SharedFoldersMultipleCheckboxField(MultipleCheckboxField):

    def __init__(self, choices=empty, *args, **kwargs):
        if choices is empty:
            choices = self._get_default_choices()
        super(SharedFoldersMultipleCheckboxField, self).__init__(choices=choices, *args, **kwargs)

    def _get_default_choices(self):
        choices = []
        try:
            from project.fal.share import Share
            choices += [(sharename, sharename) for sharename in sorted(Share().load().keys())]
        except:
            pass
        return choices


class CheckboxField(forms.BooleanField):

    def __init__(self, required=False, *args, **kwargs):
        super(CheckboxField, self).__init__(required=required, *args, **kwargs)


class TextField(forms.CharField):

    def __init__(self, max_length=1024, *args, **kwargs):
        super(TextField, self).__init__(max_length=max_length, *args, **kwargs)


class IOSerialNumberTextField(forms.CharField):

    def __init__(self,
                 max_length=12,
                 *args, **kwargs):
        super(IOSerialNumberTextField, self).__init__(max_length=max_length, *args, **kwargs)

    def clean(self, value):
        #value = super(IOSerialNumberTextField, self).clean(value)
        pattern = re.compile(r"^[0-9a-zA-Z]{%d}$" % self.max_length)
        if pattern.match(value):
            return value
        else:
            raise forms.ValidationError(self.error_messages['invalid'])
#                                                                             #
###############################################################################


###############################################################################
# IOPortal                                                                    #
class IOPortalAccountNameTextField(forms.CharField):

    def __init__(self,
                 label=_(u"e-mail address"),
                 max_length=1024,
                 *args, **kwargs):
        super(IOPortalAccountNameTextField, self).__init__(label=label, max_length=max_length, *args, **kwargs)

    def clean(self, value):
        #c = r"[!-?A-~]"
        pattern = re.compile(r"^[!-?A-~]+@([-a-z0-9]+\.)+([a-z]{2,})$")
        if pattern.match(value):
            return value
        else:
            raise forms.ValidationError(self.error_messages['invalid'])


class IOPortalPasswordField(forms.CharField):

    widget = forms.widgets.PasswordInput

    def __init__(self,
                 label=_(u"password"),
                 max_length=1024,
                 widget=None,
                 *args, **kwargs):
        widget = widget or self.widget(render_value=True)
        super(IOPortalPasswordField, self).__init__(label=label,
                                                    max_length=max_length,
                                                    widget=widget,
                                                    *args, **kwargs)


class SerialNumberOfThisProductTextField(forms.CharField):

    def __init__(self,
                 label=_(u"serial number of this product"),
                 *args, **kwargs):
        super(SerialNumberOfThisProductTextField, self).__init__(label=label, *args, **kwargs)
#                                                                             #
###############################################################################

class RemoteLink3EmailTextField(TextField):

    def __init__(self,
                 label=_(u"Remote Link 3 pincode e-mail address"),
                 *args, **kwargs):
        super(RemoteLink3EmailTextField, self).__init__(label=label, *args, **kwargs)


class RemoteLink3EnabledRadioField(EnabledRadioField):

    def __init__(self,
                 label=_(u"Remote Link 3"),
                 *args, **kwargs):
        super(RemoteLink3EnabledRadioField, self).__init__(label=label, *args, **kwargs)


class RemoteLink3TerminalAccessControlEnabledRadioField(EnabledRadioField):

    def __init__(self,
                 label=_(u"Remote Link 3 client access control"),
                 *args, **kwargs):
        super(RemoteLink3TerminalAccessControlEnabledRadioField, self).__init__(label=label, *args, **kwargs)


class RemoteLink3ExpiryAccessControlEnabledRadioField(EnabledRadioField):

    def __init__(self,
                 label=_(u"Remote Link 3 temporary access control"),
                 *args, **kwargs):
        super(RemoteLink3ExpiryAccessControlEnabledRadioField, self).__init__(label=label, *args, **kwargs)


class RemoteLink3ListenPortNumber1TextField(LocalPortNumberTextField):

    SERVICE = 'remotelink3_1'
    
    def __init__(self,
                 label=_(u"Remote Link 3 listen port %(index)s") % {'index': 1},
                 *args, **kwargs):
        super(RemoteLink3ListenPortNumber1TextField, self).__init__(label=label, *args, **kwargs)


class RemoteLink3ListenPortNumber2TextField(LocalPortNumberTextField):

    SERVICE = 'remotelink3_2'
    
    def __init__(self,
                 label=_(u"Remote Link 3 listen port %(index)s") % {'index': 2},
                 *args, **kwargs):
        super(RemoteLink3ListenPortNumber2TextField, self).__init__(label=label, *args, **kwargs)


class RemoteLink3UPnPRadioField(forms.ChoiceField):

    widget = forms.widgets.RadioSelect

    def __init__(self,
                 label=_(u"Remote Link 3 UPnP"),
                 choices=(('1', _(u"use upnp")), ('0', _(u"not use upnp"))),
                 initial='1',
                 *args, **kwargs):
        super(RemoteLink3UPnPRadioField, self).__init__(label=label, choices=choices, initial=initial, *args, **kwargs)


class RemoteLink3RemapRouterPortRadioField(forms.ChoiceField):

    widget = forms.widgets.RadioSelect

    def __init__(self,
                 label=_(u"remap router port"),
                 choices=(('1', _(u"yes")), ('0', _(u"no"))),
                 initial='0',
                 *args, **kwargs):
        super(RemoteLink3RemapRouterPortRadioField, self).__init__(label=label, choices=choices, initial=initial, *args, **kwargs)


class RemoteLink3RouterPortNumber1TextField(RemotePortNumberTextField):

    def __init__(self,
                 label=_(u"Remote Link 3 router port %(index)s") % {'index': 1},
                 *args, **kwargs):
        super(RemoteLink3RouterPortNumber1TextField, self).__init__(label=label, *args, **kwargs)


class RemoteLink3RouterPortNumber2TextField(RemotePortNumberTextField):

    def __init__(self,
                 label=_(u"Remote Link 3 router port %(index)s") % {'index': 2},
                 *args, **kwargs):
        super(RemoteLink3RouterPortNumber2TextField, self).__init__(label=label, *args, **kwargs)


class RemoteLink3EnabledSharedFoldersMultipleCheckboxField(SharedFoldersMultipleCheckboxField):

    def __init__(self,
                 label=_(u"Remote Link 3"),
                 *args, **kwargs):
        super(RemoteLink3EnabledSharedFoldersMultipleCheckboxField, self).__init__(label=label, *args, **kwargs)


class RemoteLink3TerminalAccessControlSharedFoldersMultipleCheckboxField(SharedFoldersMultipleCheckboxField):

    def __init__(self,
                 label=_(u"Remote Link 3 client access control"),
                 *args, **kwargs):
        super(RemoteLink3TerminalAccessControlSharedFoldersMultipleCheckboxField, self).__init__(label=label, *args, **kwargs)


class RemoteLink3ClientEntrysMultipleCheckboxField(MultipleCheckboxField):

    def __init__(self,
                 label=_(u"client"),
                 choices=empty,
                 *args, **kwargs):
        if choices is empty:
            choices = self._get_default_choices()
        super(RemoteLink3ClientEntrysMultipleCheckboxField, self).__init__(label=label, choices=choices, *args, **kwargs)

    def _get_default_choices(self):
        choices = []
        try:
            pass  # FIXME
        except:
            pass
        return choices


class RemoteLink3TerminalLabelTextField(TextField):

    def __init__(self,
                 label=_(u"Remote Link 3 terminal label"),
                 max_length=30,
                 *args, **kwargs):
        super(RemoteLink3TerminalLabelTextField, self).__init__(label=label, *args, **kwargs)


class RemoteLink3TerminalOwnersEmailTextField(TextField):

    def __init__(self,
                 label=_(u"Remote Link 3 terminal owners e-mail address"),
                 *args, **kwargs):
        super(RemoteLink3TerminalOwnersEmailTextField, self).__init__(label=label, *args, **kwargs)


class RemoteLink3ExpirationEntrysMultipleCheckboxField(MultipleCheckboxField):

    def __init__(self,
                 label=_(u"Remote Link 3 PIN code"),
                 choices=empty,
                 *args, **kwargs):
        if choices is empty:
            choices = self._get_default_choices()
        super(RemoteLink3ExpirationEntrysMultipleCheckboxField, self).__init__(label=label, choices=choices, *args, **kwargs)

    def _get_default_choices(self):
        choices = []
        try:
            pass  # FIXME
        except:
            pass
        return choices

"""
class RemoteLink3ConnectionExpiryField(forms.ChoiceField):
    def __init__(self, **kwargs):
        kwargs["choices"] = kwargs.get("choices", (
            (u"0", u"無期限"),
            (u"1", u"１日"),
            (u"7", u"１週間"),
            (u"30", u"１ヶ月"),
            (u"365", u"１年"),
        ))
        super(RemoteLink3ConnectionExpiryField, self).__init__(**kwargs)
"""


class RemoteLink3ConnectionExpiryEnabledField(EnabledRadioField):
    def __init__(self, **kwargs):
        kwargs["label"] = u"再接続期限機能"
        super(RemoteLink3ConnectionExpiryEnabledField, self).__init__(**kwargs)


class RemoteLink3ConnectionExpiryField(forms.IntegerField):
    def __init__(self, **kwargs):
        kwargs['label'] = u"再接続期限"
        kwargs['min_value'] = kwargs.get('min_value', 1)
        kwargs['max_value'] = kwargs.get('max_value', 365)
        super(RemoteLink3ConnectionExpiryField, self).__init__(**kwargs)
